from functools import partial
from typing import Callable, Optional

import click
from validators import url


# Shamelessly inspired by click_params
class CustomParamType(click.ParamType):
    name: Optional[str] = None


class ValidatorParamType(CustomParamType):
    """This class is intended to be inherit by classes using validators functions."""

    def __init__(self, callback: Callable, name: str = None):  # type: ignore
        self._callback = callback
        self._name = name or self.name
        self._error_message = "{value} is not a valid %s" % self._name

    def convert(self, value, param, ctx):
        if not self._callback(value):
            self.fail(self._error_message.format(value=value), param, ctx)
        return value

    def __repr__(self):
        return self.name.upper()


class UrlParamType(ValidatorParamType):
    name = "url"

    def __init__(self, public: bool = False):
        super().__init__(callback=partial(url, public=public))  # type: ignore


ValidUrl = UrlParamType()
